@@ -1,19 +1,24 @@
+/* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /*
  * Copyright © 2014 Endless Mobile, Inc.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ * IN THE SOFTWARE.
  *
  * Authored By: Sam Spilsbury <sam@endlessm.com>
  */
@@ -25,6 +30,7 @@
 
 #include "coverage.h"
 #include "coverage-internal.h"
+#include "global.h"
 #include "importer.h"
 #include "jsapi-util-args.h"
 #include "util/error.h"
@@ -407,7 +413,10 @@ get_array_from_js_value(JSContext             *context,
     g_return_val_if_fail(out_array != NULL, false);
     g_return_val_if_fail(*out_array == NULL, false);
 
-    if (!JS_IsArrayObject(context, value)) {
+    bool is_array;
+    if (!JS_IsArrayObject(context, value, &is_array))
+        return false;
+    if (!is_array) {
         g_critical("Returned object from is not an array");
         return false;
     }
@@ -422,21 +431,17 @@ get_array_from_js_value(JSContext             *context,
     if (element_clear_func)
         g_array_set_clear_func(c_side_array, element_clear_func);
 
-    if (JS_GetArrayLength(context, js_array, &js_array_len)) {
-        uint32_t i = 0;
-        JS::RootedValue element(context);
-        for (; i < js_array_len; ++i) {
-            if (!JS_GetElement(context, js_array, i, &element)) {
-                g_array_unref(c_side_array);
-                gjs_throw(context, "Failed to get function names array element %d", i);
-                return false;
-            }
+    if (!JS_GetArrayLength(context, js_array, &js_array_len)) {
+        g_array_unref(c_side_array);
+        return false;
+    }
 
-            if (!(inserter(c_side_array, context, element))) {
-                g_array_unref(c_side_array);
-                gjs_throw(context, "Failed to convert array element %d", i);
-                return false;
-            }
+    JS::RootedValue element(context);
+    for (uint32_t i = 0; i < js_array_len; ++i) {
+        if (!JS_GetElement(context, js_array, i, &element) ||
+            !inserter(c_side_array, context, element)) {
+            g_array_unref(c_side_array);
+            return false;
         }
     }
 
@@ -493,11 +498,11 @@ get_executed_lines_for(JSContext        *context,
 
 static void
 init_covered_function(GjsCoverageFunction *function,
-                      char                *key,
+                      const char          *key,
                       unsigned int        line_number,
                       unsigned int        hit_count)
 {
-    function->key = key;
+    function->key = g_strdup(key);
     function->line_number = line_number;
     function->hit_count = hit_count;
 }
@@ -517,12 +522,12 @@ get_hit_count_and_line_data(JSContext       *cx,
                             int32_t         *line)
 {
     JS::RootedId hit_count_name(cx, gjs_intern_string_to_id(cx, "hitCount"));
-    if (!gjs_object_require_property(cx, obj, "function element",
+    if (!gjs_object_require_property(cx, obj, description,
                                      hit_count_name, hit_count))
         return false;
 
     JS::RootedId line_number_name(cx, gjs_intern_string_to_id(cx, "line"));
-    return gjs_object_require_property(cx, obj, "function_element",
+    return gjs_object_require_property(cx, obj, description,
                                        line_number_name, line);
 }
 
@@ -543,7 +548,7 @@ convert_and_insert_function_decl(GArray         *array,
                                      &function_name_property_value))
         return false;
 
-    char *utf8_string;
+    GjsAutoJSChar utf8_string(context);
 
     if (function_name_property_value.isString()) {
         if (!gjs_string_to_utf8(context,
@@ -552,9 +557,7 @@ convert_and_insert_function_decl(GArray         *array,
             gjs_throw(context, "Failed to convert function_name to string");
             return false;
         }
-    } else if (function_name_property_value.isNull()) {
-        utf8_string = NULL;
-    } else {
+    } else if (!function_name_property_value.isNull()) {
         gjs_throw(context, "Unexpected type for function_name");
         return false;
     }
@@ -869,14 +872,14 @@ get_covered_files(GjsCoverage *coverage)
     files = g_new0(char *, n_files + 1);
     JS::RootedValue element(context);
     for (uint32_t i = 0; i < n_files; i++) {
-        char *file;
+        GjsAutoJSChar file(context);
         if (!JS_GetElement(context, files_obj, i, &element))
             goto error;
 
         if (!gjs_string_to_utf8(context, element, &file))
             goto error;
 
-        files[i] = file;
+        files[i] = file.copy();
     }
 
     files[n_files] = NULL;
@@ -963,44 +966,16 @@ gjs_get_file_checksum(GFile *file)
     return checksum;
 }
 
-/* The binary data for the cache has the following structure:
- *
- * {
- *     array [ tuple {
- *         string filename;
- *         string? checksum;
- *         tuple? {
- *             mtime_sec;
- *             mtime_usec;
- *         }
- *         array [
- *             int line;
- *         ] executable lines;
- *         array [ tuple {
- *             int branch_point;
- *             array [
- *                 int line;
- *             ] exits;
- *         } branch_info ] branches;
- *         array [ tuple {
- *             int line;
- *             string key;
- *         } function ] functions;
- *     } file ] files;
- */
-const char *COVERAGE_STATISTICS_CACHE_BINARY_DATA_TYPE = "a(sm(xx)msaia(iai)a(is))";
-
 GBytes *
 gjs_serialize_statistics(GjsCoverage *coverage)
 {
     GjsCoveragePrivate *priv = (GjsCoveragePrivate *) gjs_coverage_get_instance_private(coverage);
     JSContext *js_context = (JSContext *) gjs_context_get_native_context(priv->context);
-    JSRuntime *js_runtime = JS_GetRuntime(js_context);
 
     JSAutoRequest ar(js_context);
     JSAutoCompartment ac(js_context, priv->coverage_statistics);
     JS::RootedObject rooted_priv(js_context, priv->coverage_statistics);
-    JS::RootedValue string_value_return(js_runtime);
+    JS::RootedValue string_value_return(js_context);
 
     if (!JS_CallFunctionName(js_context, rooted_priv, "stringify",
                              JS::HandleValueArray::empty(),
@@ -1013,7 +988,7 @@ gjs_serialize_statistics(GjsCoverage *coverage)
         return NULL;
 
     /* Free'd by g_bytes_new_take */
-    char *statistics_as_json_string = NULL;
+    GjsAutoJSChar statistics_as_json_string(js_context);
 
     if (!gjs_string_to_utf8(js_context,
                             string_value_return.get(),
@@ -1022,8 +997,12 @@ gjs_serialize_statistics(GjsCoverage *coverage)
         return NULL;
     }
 
-    return g_bytes_new_take((guint8 *) statistics_as_json_string,
-                            strlen(statistics_as_json_string));
+    int json_string_len = strlen(statistics_as_json_string);
+    auto json_bytes =
+        reinterpret_cast<uint8_t*>(statistics_as_json_string.copy());
+
+    return g_bytes_new_take(json_bytes,
+                            json_string_len);
 }
 
 static JSString *
@@ -1068,7 +1047,7 @@ gjs_deserialize_cache_to_object(GjsCoverage *coverage,
     JSContext *context = (JSContext *) gjs_context_get_native_context(priv->context);
     JSAutoRequest ar(context);
     JSAutoCompartment ac(context, priv->coverage_statistics);
-    JS::RootedObject global_object(JS_GetRuntime(context),
+    JS::RootedObject global_object(context,
                                    JS_GetGlobalForObject(context, priv->coverage_statistics));
     return gjs_deserialize_cache_to_object_for_compartment(context, global_object, cache_data);
 }
@@ -1086,7 +1065,7 @@ gjs_fetch_statistics_from_js(GjsCoverage *coverage,
     g_array_set_clear_func(file_statistics_array,
                            gjs_coverage_statistics_file_statistics_clear);
 
-    JS::RootedObject rooted_coverage_statistics(JS_GetRuntime(js_context),
+    JS::RootedObject rooted_coverage_statistics(js_context,
                                                 priv->coverage_statistics);
 
     char **file_iter = coverage_files;
@@ -1257,24 +1236,6 @@ gjs_coverage_init(GjsCoverage *self)
 {
 }
 
-static JSClass coverage_global_class = {
-    "GjsCoverageGlobal",
-    JSCLASS_GLOBAL_FLAGS_WITH_SLOTS(GJS_GLOBAL_SLOT_LAST) |
-    JSCLASS_IMPLEMENTS_BARRIERS,
-    NULL,  /* addProperty */
-    NULL,  /* deleteProperty */
-    NULL,  /* getProperty */
-    NULL,  /* setProperty */
-    NULL,  /* enumerate */
-    NULL,  /* resolve */
-    NULL,  /* convert */
-    NULL,  /* finalize */
-    NULL,  /* call */
-    NULL,  /* hasInstance */
-    NULL,  /* construct */
-    JS_GlobalObjectTraceHook
-};
-
 static bool
 gjs_context_eval_file_in_compartment(GjsContext      *context,
                                      const char      *filename,
@@ -1311,12 +1272,12 @@ gjs_context_eval_file_in_compartment(GjsContext      *context,
            .setFileAndLine(filename, start_line_number)
            .setSourceIsLazy(true);
     JS::RootedScript compiled_script(js_context);
-    if (!JS::Compile(js_context, compartment_object, options, stripped_script,
-                     script_len, &compiled_script))
+    if (!JS::Compile(js_context, options, stripped_script, script_len,
+                     &compiled_script))
         return false;
 
-    if (!JS::CloneAndExecuteScript(js_context, compartment_object,
-                                   compiled_script)) {
+    JS::RootedValue dummy_rval(js_context);
+    if (!JS::CloneAndExecuteScript(js_context, compiled_script, &dummy_rval)) {
         g_free(script);
         gjs_log_exception(js_context);
         g_set_error(error, GJS_ERROR, GJS_ERROR_FAILED, "Failed to evaluate %s", filename);
@@ -1334,7 +1295,7 @@ coverage_log(JSContext *context,
              JS::Value *vp)
 {
     JS::CallArgs argv = JS::CallArgsFromVp (argc, vp);
-    char *s;
+    GjsAutoJSChar s(context);
     JSExceptionState *exc_state;
 
     if (argc != 1) {
@@ -1354,11 +1315,11 @@ coverage_log(JSContext *context,
      * could not be converted to string */
     exc_state = JS_SaveExceptionState(context);
     JS::RootedString jstr(context, JS::ToString(context, argv[0]));
-    if (jstr != NULL)
+    if (jstr)
         argv[0].setString(jstr);  // GC root
     JS_RestoreExceptionState(context, exc_state);
 
-    if (jstr == NULL) {
+    if (!jstr) {
         g_message("JS LOG: <cannot convert value to string>");
         return true;
     }
@@ -1367,8 +1328,7 @@ coverage_log(JSContext *context,
         return false;
     }
 
-    g_message("JS COVERAGE MESSAGE: %s", s);
-    g_free(s);
+    g_message("JS COVERAGE MESSAGE: %s", s.get());
 
     argv.rval().setUndefined();
     return true;
@@ -1377,17 +1337,14 @@ coverage_log(JSContext *context,
 static GFile *
 get_file_from_call_args_filename(JSContext    *context,
                                  JS::CallArgs &args) {
-    char *filename = NULL;
+    GjsAutoJSChar filename(context);
 
     if (!gjs_parse_call_args(context, "getFileContents", args, "s",
                              "filename", &filename))
         return NULL;
 
     /* path could be a resource, so use g_file_new_for_commandline_arg. */
-    GFile *file = g_file_new_for_commandline_arg(filename);
-
-    g_free(filename);
-    return file;
+    return g_file_new_for_commandline_arg(filename);
 }
 
 static bool
@@ -1409,7 +1366,7 @@ coverage_get_file_modification_time(JSContext *context,
         mtime_values_array[1].setInt32(mtime.tv_usec);
         JS::RootedObject array_obj(context,
             JS_NewArrayObject(context, mtime_values_array));
-        if (array_obj == NULL)
+        if (!array_obj)
             goto out;
         args.rval().setObject(*array_obj);
     } else {
@@ -1429,7 +1386,6 @@ coverage_get_file_checksum(JSContext *context,
                            JS::Value *vp)
 {
     JS::CallArgs args = JS::CallArgsFromVp(argc, vp);
-    JSRuntime *runtime = JS_GetRuntime(context);
     GFile *file = get_file_from_call_args_filename(context, args);
 
     if (!file)
@@ -1445,9 +1401,7 @@ coverage_get_file_checksum(JSContext *context,
         return false;
     }
 
-    JS::RootedString rooted_checksum(runtime, JS_NewStringCopyZ(context,
-                                                                checksum));
-    args.rval().setString(rooted_checksum);
+    args.rval().setString(JS_NewStringCopyZ(context, checksum));
 
     g_object_unref(file);
     g_free(checksum);
@@ -1506,7 +1460,8 @@ coverage_statistics_tracer(JSTracer *trc, void *data)
     GjsCoverage *coverage = (GjsCoverage *) data;
     GjsCoveragePrivate *priv = (GjsCoveragePrivate *) gjs_coverage_get_instance_private(coverage);
 
-    JS_CallObjectTracer(trc, &priv->coverage_statistics, "coverage_statistics");
+    JS::TraceEdge<JSObject *>(trc, &priv->coverage_statistics,
+                              "coverage_statistics");
 }
 
 /* This function is mainly used in the tests in order to fiddle with
@@ -1525,10 +1480,7 @@ gjs_run_script_in_coverage_compartment(GjsCoverage *coverage,
     options.setUTF8(true);
 
     JS::RootedValue rval(js_context);
-    JS::RootedObject global(js_context,
-        JS_GetGlobalForObject(js_context, priv->coverage_statistics));
-    if (!JS::Evaluate(js_context, global, options, script, strlen(script),
-                      &rval)) {
+    if (!JS::Evaluate(js_context, options, script, strlen(script), &rval)) {
         gjs_log_exception(js_context);
         g_warning("Failed to evaluate <coverage_modifier>");
         return false;
@@ -1548,7 +1500,7 @@ gjs_inject_value_into_coverage_compartment(GjsCoverage     *coverage,
     JSAutoRequest ar(js_context);
     JSAutoCompartment ac(js_context, priv->coverage_statistics);
 
-    JS::RootedObject coverage_global_scope(JS_GetRuntime(js_context),
+    JS::RootedObject coverage_global_scope(js_context,
                                            JS_GetGlobalForObject(js_context, priv->coverage_statistics));
 
     if (!JS_SetProperty(js_context, coverage_global_scope, property,
@@ -1560,29 +1512,6 @@ gjs_inject_value_into_coverage_compartment(GjsCoverage     *coverage,
     return true;
 }
 
-/* Gets the root import and wraps it into a cross-compartment
- * object so that it can be used in the debugger compartment */
-static JSObject *
-gjs_wrap_root_importer_in_compartment(JSContext *context,
-                                      JS::HandleObject compartment)
-{
-    JSAutoRequest ar(context);
-    JSAutoCompartment ac(context, compartment);
-    JS::RootedValue importer (JS_GetRuntime(context),
-                              gjs_get_global_slot(context,
-                                                  GJS_GLOBAL_SLOT_IMPORTS));
-
-    g_assert (!importer.isUndefined());
-
-    JS::RootedObject wrapped_importer(JS_GetRuntime(context),
-                                      importer.toObjectOrNull());
-    if (!JS_WrapObject(context, &wrapped_importer)) {
-        return NULL;
-    }
-
-    return wrapped_importer;
-}
-
 static bool
 bootstrap_coverage(GjsCoverage *coverage)
 {
@@ -1596,10 +1525,9 @@ bootstrap_coverage(GjsCoverage *coverage)
 
     JSObject *debuggee = gjs_get_import_global(context);
     JS::CompartmentOptions options;
-    options.setVersion(JSVERSION_LATEST);
-    JS::RootedObject debugger_compartment(JS_GetRuntime(context),
-        JS_NewGlobalObject(context, &coverage_global_class, NULL,
-                           JS::FireOnNewGlobalHook, options));
+    options.behaviors().setVersion(JSVERSION_LATEST);
+    JS::RootedObject debugger_compartment(context,
+                                          gjs_create_global_object(context));
     {
         JSAutoCompartment compartment(context, debugger_compartment);
         JS::RootedObject debuggeeWrapper(context, debuggee);
@@ -1615,35 +1543,9 @@ bootstrap_coverage(GjsCoverage *coverage)
             return false;
         }
 
-        if (!JS_InitStandardClasses(context, debugger_compartment)) {
-            gjs_throw(context, "Failed to init standard classes");
-            return false;
-        }
-
-        if (!JS_InitReflect(context, debugger_compartment)) {
-            gjs_throw(context, "Failed to init Reflect");
-            return false;
-        }
-
-        if (!JS_DefineDebuggerObject(context, debugger_compartment)) {
-            gjs_throw(context, "Failed to init Debugger");
-            return false;
-        }
-
-        JS::RootedObject wrapped_importer(JS_GetRuntime(context),
-                                          gjs_wrap_root_importer_in_compartment(context,
-                                                                                debugger_compartment));;
-
-        if (!wrapped_importer) {
-            gjs_throw(context, "Failed to wrap root importer in debugger compartment");
-            return false;
-        }
-
-        /* Now copy the global root importer (which we just created,
-         * if it didn't exist) to our global object
-         */
-        if (!gjs_define_root_importer_object(context, debugger_compartment, wrapped_importer)) {
-            gjs_throw(context, "Failed to set 'imports' on debugger compartment");
+        if (!gjs_define_global_properties(context, debugger_compartment)) {
+            gjs_throw(context, "Failed to define global properties on debugger "
+                      "compartment");
             return false;
         }
 
@@ -1700,9 +1602,7 @@ bootstrap_coverage(GjsCoverage *coverage)
         }
 
         /* Add a tracer, as suggested by jdm on #jsapi */
-        JS_AddExtraGCRootsTracer(JS_GetRuntime(context),
-                                 coverage_statistics_tracer,
-                                 coverage);
+        JS_AddExtraGCRootsTracer(context, coverage_statistics_tracer, coverage);
 
         priv->coverage_statistics = coverage_statistics;
     }
@@ -1724,11 +1624,6 @@ gjs_coverage_constructed(GObject *object)
         priv->cache = g_file_new_for_path(".internal-gjs-coverage-cache");
     }
 
-    /* We now enable Ion and BaselineJIT in coverage mode. See the comment
-     * in gjs/runtime.cpp:gjs_clear_thread_runtime for some important
-     * information regarding runtime lifecycle management and garbage collection
-     * bugs in js24 */
-
     if (!bootstrap_coverage(coverage)) {
         JSContext *context = static_cast<JSContext *>(gjs_context_get_native_context(priv->context));
         JSAutoCompartment compartment(context, gjs_get_import_global(context));
@@ -1777,7 +1672,7 @@ gjs_clear_js_side_statistics_from_coverage_object(GjsCoverage *coverage)
         JSAutoRequest ar(js_context);
         JSAutoCompartment ac(js_context, priv->coverage_statistics);
         JS::RootedObject rooted_priv(js_context, priv->coverage_statistics);
-        JS::RootedValue rval(JS_GetRuntime(js_context));
+        JS::RootedValue rval(js_context);
         if (!JS_CallFunctionName(js_context, rooted_priv, "deactivate",
                                  JS::HandleValueArray::empty(), &rval)) {
             gjs_log_exception(js_context);
@@ -1786,8 +1681,7 @@ gjs_clear_js_side_statistics_from_coverage_object(GjsCoverage *coverage)
 
         /* Remove GC roots trace after we've decomissioned the object
          * and no longer need it to be traced here. */
-        JS_RemoveExtraGCRootsTracer(JS_GetRuntime(js_context),
-                                    coverage_statistics_tracer,
+        JS_RemoveExtraGCRootsTracer(js_context, coverage_statistics_tracer,
                                     coverage);
 
         priv->coverage_statistics = NULL;
