@@ -30,6 +30,7 @@
 #include "jsapi-class.h"
 #include "jsapi-wrapper.h"
 #include "mem.h"
+#include "module.h"
 #include "native.h"
 
 #include <gio/gio.h>
@@ -62,6 +63,9 @@ static const JSClass gjs_importer_class = *js::Jsvalify(&gjs_importer_real_class
 
 GJS_DEFINE_PRIV_FROM_JS(Importer, gjs_importer_class)
 
+static JSObject *gjs_define_importer(JSContext *, JS::HandleObject,
+    const char *, const char **, bool);
+
 static bool
 importer_to_string(JSContext *cx,
                    unsigned   argc,
@@ -75,7 +79,7 @@ importer_to_string(JSContext *cx,
                                  &module_path))
         return false;
 
-    char *path = NULL;
+    GjsAutoJSChar path(cx);
     GjsAutoChar output;
 
     if (module_path.isNull()) {
@@ -83,11 +87,10 @@ importer_to_string(JSContext *cx,
     } else {
         if (!gjs_string_to_utf8(cx, module_path, &path))
             return false;
-        output = g_strdup_printf("[%s %s]", klass->name, path);
+        output = g_strdup_printf("[%s %s]", klass->name, path.get());
     }
 
     args.rval().setString(JS_NewStringCopyZ(cx, output));
-    g_free(path);
     return true;
 }
 
@@ -100,6 +103,12 @@ define_meta_properties(JSContext       *context,
 {
     bool parent_is_module;
 
+    /* For these meta-properties, don't set ENUMERATE since we wouldn't want to
+     * copy these symbols to any other object for example. RESOLVING is used to
+     * make sure we don't try to invoke a "resolve" operation, since this
+     * function may be called from inside one. */
+    unsigned attrs = JSPROP_READONLY | JSPROP_PERMANENT | JSPROP_RESOLVING;
+
     /* We define both __moduleName__ and __parentModule__ to null
      * on the root importer
      */
@@ -112,19 +121,16 @@ define_meta_properties(JSContext       *context,
 
     if (full_path != NULL) {
         JS::RootedString file(context, JS_NewStringCopyZ(context, full_path));
-        if (!JS_DefineProperty(context, module_obj, "__file__", file,
-                               /* don't set ENUMERATE since we wouldn't want to copy
-                                * this symbol to any other object for example.
-                                */
-                               JSPROP_READONLY | JSPROP_PERMANENT))
+        if (!JS_DefineProperty(context, module_obj, "__file__", file, attrs))
             return false;
     }
 
-    /* Null is used instead of undefined to make sure we don't try to invoke
-     * a "resolve" operation. */
+    /* Null is used instead of undefined for backwards compatibility with code
+     * that explicitly checks for null. */
     JS::RootedValue module_name_val(context, JS::NullValue());
     JS::RootedValue parent_module_val(context, JS::NullValue());
     JS::RootedValue module_path(context, JS::NullValue());
+    JS::RootedValue to_string_tag(context);
     if (parent_is_module) {
         module_name_val.setString(JS_NewStringCopyZ(context, module_name));
         parent_module_val.setObject(*parent);
@@ -139,32 +145,37 @@ define_meta_properties(JSContext       *context,
         if (parent_module_path.isNull()) {
             module_path_buf = g_strdup(module_name);
         } else {
-            char *parent_path = NULL;
+            GjsAutoJSChar parent_path(context);
             if (!gjs_string_to_utf8(context, parent_module_path, &parent_path))
                 return false;
-            module_path_buf = g_strdup_printf("%s.%s", parent_path, module_name);
-            g_free(parent_path);
+            module_path_buf = g_strdup_printf("%s.%s", parent_path.get(), module_name);
         }
         module_path.setString(JS_NewStringCopyZ(context, module_path_buf));
+
+        GjsAutoChar to_string_tag_buf = g_strdup_printf("GjsModule %s",
+                                                        module_path_buf.get());
+        to_string_tag.setString(JS_NewStringCopyZ(context, to_string_tag_buf));
+    } else {
+        to_string_tag.setString(JS_AtomizeString(context, "GjsModule"));
     }
 
-    /* don't set ENUMERATE since we wouldn't want to copy these symbols to any
-     * other object for example. */
     if (!JS_DefineProperty(context, module_obj,
-                           "__moduleName__", module_name_val,
-                           JSPROP_READONLY | JSPROP_PERMANENT))
+                           "__moduleName__", module_name_val, attrs))
         return false;
 
     if (!JS_DefineProperty(context, module_obj,
-                           "__parentModule__", parent_module_val,
-                           JSPROP_READONLY | JSPROP_PERMANENT))
+                           "__parentModule__", parent_module_val, attrs))
         return false;
 
     if (!JS_DefineProperty(context, module_obj, "__modulePath__", module_path,
-                           JSPROP_READONLY | JSPROP_PERMANENT))
+                           attrs))
         return false;
 
-    return true;
+    JS::RootedId to_string_tag_name(context,
+        SYMBOL_TO_JSID(JS::GetWellKnownSymbol(context,
+                                              JS::SymbolCode::toStringTag)));
+    return JS_DefinePropertyById(context, module_obj, to_string_tag_name,
+                                 to_string_tag, attrs);
 }
 
 static bool
@@ -184,51 +195,29 @@ import_directory(JSContext       *context,
      * this always succeeds.
      */
     importer = gjs_define_importer(context, obj, name, full_paths, false);
-    return importer != NULL;
+    return importer != nullptr;
 }
 
-static bool
-define_import(JSContext       *context,
-              JS::HandleObject obj,
-              JS::HandleObject module_obj,
-              const char      *name)
-{
-    if (!JS_DefineProperty(context, obj, name, module_obj,
-                           GJS_MODULE_PROP_FLAGS & ~JSPROP_PERMANENT)) {
-        gjs_debug(GJS_DEBUG_IMPORTER,
-                  "Failed to define '%s' in importer",
-                  name);
-        return false;
-    }
-
-    return true;
-}
-
-/* Make the property we set in define_import permament;
+/* Make the property we set in gjs_module_import() permanent;
  * we do this after the import succesfully completes.
  */
 static bool
 seal_import(JSContext       *cx,
             JS::HandleObject obj,
+            JS::HandleId     id,
             const char      *name)
 {
-    JS::Rooted<JSPropertyDescriptor> descr(cx);
+    JS::Rooted<JS::PropertyDescriptor> descr(cx);
 
-    if (!JS_GetOwnPropertyDescriptor(cx, obj, name, &descr) ||
-        descr.object() == NULL) {
+    if (!JS_GetOwnPropertyDescriptorById(cx, obj, id, &descr) || !descr.object()) {
         gjs_debug(GJS_DEBUG_IMPORTER,
                   "Failed to get attributes to seal '%s' in importer",
                   name);
         return false;
     }
 
-    /* COMPAT: in mozjs45 use .setConfigurable(false) and the form of
-     * JS_DefineProperty that takes the JSPropertyDescriptor directly */
-
-    if (!JS_DefineProperty(cx, descr.object(), name, descr.value(),
-                           descr.attributes() | JSPROP_PERMANENT,
-                           JS_PROPERTYOP_GETTER(descr.getter()),
-                           JS_PROPERTYOP_SETTER(descr.setter()))) {
+    descr.setConfigurable(false);
+    if (!JS_DefinePropertyById(cx, descr.object(), id, descr)) {
         gjs_debug(GJS_DEBUG_IMPORTER,
                   "Failed to redefine attributes to seal '%s' in importer",
                   name);
@@ -272,30 +261,6 @@ cancel_import(JSContext       *context,
     }
 }
 
-static bool
-module_to_string(JSContext *cx,
-                 unsigned   argc,
-                 JS::Value *vp)
-{
-    GJS_GET_THIS(cx, argc, vp, args, module);
-
-    JS::RootedValue module_path(cx);
-    if (!gjs_object_get_property(cx, module, GJS_STRING_MODULE_PATH,
-                                 &module_path))
-        return false;
-
-    g_assert(!module_path.isNull());
-
-    char *path = NULL;
-    if (!gjs_string_to_utf8(cx, module_path, &path))
-        return false;
-    GjsAutoChar output = g_strdup_printf("[GjsModule %s]", path);
-
-    args.rval().setString(JS_NewStringCopyZ(cx, output));
-    g_free(path);
-    return true;
-}
-
 static bool
 import_native_file(JSContext       *context,
                    JS::HandleObject obj,
@@ -311,10 +276,6 @@ import_native_file(JSContext       *context,
     if (!define_meta_properties(context, module_obj, NULL, name, obj))
         return false;
 
-    if (!JS_DefineFunction(context, module_obj, "toString", module_to_string,
-                           0, 0))
-        return false;
-
     if (JS_IsExceptionPending(context)) {
         /* I am not sure whether this can happen, but if it does we want to trap it.
          */
@@ -330,10 +291,9 @@ import_native_file(JSContext       *context,
 }
 
 static bool
-import_file(JSContext       *context,
-            const char      *name,
-            GFile           *file,
-            JS::HandleObject module_obj)
+import_module_init(JSContext       *context,
+                   GFile           *file,
+                   JS::HandleObject module_obj)
 {
     bool ret = false;
     char *script = NULL;
@@ -370,17 +330,6 @@ import_file(JSContext       *context,
     return ret;
 }
 
-static gboolean
-is_extension_module (const gchar *path)
-{
-    if (g_strstr_len (path, -1, "applets") ||
-        g_strstr_len (path, -1, "desklets") ||
-        g_strstr_len (path, -1, "extensions"))
-        return TRUE;
-
-    return FALSE;
-}
-
 static JSObject *
 load_module_init(JSContext       *context,
                  JS::HandleObject in_object,
@@ -389,8 +338,7 @@ load_module_init(JSContext       *context,
     bool found;
 
     /* First we check if js module has already been loaded  */
-    if (!is_extension_module (full_path) &&
-        gjs_object_has_property(context, in_object, GJS_STRING_MODULE_INIT,
+    if (gjs_object_has_property(context, in_object, GJS_STRING_MODULE_INIT,
                                 &found) && found) {
         JS::RootedValue module_obj_val(context);
         if (gjs_object_get_property(context, in_object,
@@ -402,7 +350,7 @@ load_module_init(JSContext       *context,
 
     JS::RootedObject module_obj(context, JS_NewPlainObject(context));
     GjsAutoUnref<GFile> file = g_file_new_for_commandline_arg(full_path);
-    if (!import_file (context, "__init__", file, module_obj))
+    if (!import_module_init(context, file, module_obj))
         return module_obj;
 
     gjs_object_define_property(context, in_object,
@@ -421,15 +369,16 @@ load_module_elements(JSContext        *cx,
     size_t ix, length;
     JS::RootedObject module_obj(cx, load_module_init(cx, in_object, init_path));
 
-    if (module_obj == NULL)
+    if (!module_obj)
         return;
 
-    JS::AutoIdArray ids(cx, JS_Enumerate(cx, module_obj));
-    if (!ids)
+    JS::Rooted<JS::IdVector> ids(cx, cx);
+    if (!JS_Enumerate(cx, module_obj, &ids))
         return;
 
     for (ix = 0, length = ids.length(); ix < length; ix++)
-        prop_ids.append(ids[ix]);
+        if (!prop_ids.append(ids[ix]))
+            g_error("Unable to append to vector");
 }
 
 /* If error, returns false. If not found, returns true but does not touch
@@ -471,29 +420,23 @@ import_symbol_from_init_js(JSContext       *cx,
 static bool
 import_file_on_module(JSContext       *context,
                       JS::HandleObject obj,
+                      JS::HandleId     id,
                       const char      *name,
                       GFile           *file)
 {
     bool retval = false;
     char *full_path = NULL;
 
-    JS::RootedObject module_obj(context, JS_NewPlainObject(context));
-
-    if (!define_import(context, obj, module_obj, name))
-        goto out;
-
-    if (!import_file(context, name, file, module_obj))
+    JS::RootedObject module_obj(context,
+        gjs_module_import(context, obj, id, name, file));
+    if (!module_obj)
         goto out;
 
     full_path = g_file_get_parse_name (file);
     if (!define_meta_properties(context, module_obj, full_path, name, obj))
         goto out;
 
-    if (!JS_DefineFunction(context, module_obj, "toString", module_to_string,
-                           0, 0))
-        goto out;
-
-    if (!seal_import(context, obj, name))
+    if (!seal_import(context, obj, id, name))
         goto out;
 
     retval = true;
@@ -510,24 +453,25 @@ static bool
 do_import(JSContext       *context,
           JS::HandleObject obj,
           Importer        *priv,
+          JS::HandleId     id,
           const char      *name)
 {
     char *filename;
     char *full_path;
-    char *dirname = NULL;
     JS::RootedObject search_path(context);
     guint32 search_path_len;
     guint32 i;
-    bool result;
+    bool result, exists, is_array;
     GPtrArray *directories;
     GFile *gfile;
-    bool exists;
 
     if (!gjs_object_require_property(context, obj, "importer",
                                      GJS_STRING_SEARCH_PATH, &search_path))
         return false;
 
-    if (!JS_IsArrayObject(context, search_path)) {
+    if (!JS_IsArrayObject(context, search_path, &is_array))
+        return false;
+    if (!is_array) {
         gjs_throw(context, "searchPath property on importer is not an array");
         return false;
     }
@@ -556,6 +500,8 @@ do_import(JSContext       *context,
     }
 
     for (i = 0; i < search_path_len; ++i) {
+        GjsAutoJSChar dirname(context);
+
         elem.setUndefined();
         if (!JS_GetElement(context, search_path, i, &elem)) {
             /* this means there was an exception, while elem.isUndefined()
@@ -572,9 +518,6 @@ do_import(JSContext       *context,
             goto out;
         }
 
-        g_free(dirname);
-        dirname = NULL;
-
         if (!gjs_string_to_utf8(context, elem, &dirname))
             goto out; /* Error message already set */
 
@@ -628,13 +571,13 @@ do_import(JSContext       *context,
         if (!exists) {
             gjs_debug(GJS_DEBUG_IMPORTER,
                       "JS import '%s' not found in %s",
-                      name, dirname);
+                      name, dirname.get());
 
             g_object_unref(gfile);
             continue;
         }
 
-        if (import_file_on_module (context, obj, name, gfile)) {
+        if (import_file_on_module(context, obj, id, name, gfile)) {
             gjs_debug(GJS_DEBUG_IMPORTER,
                       "successfully imported module '%s'", name);
             result = true;
@@ -677,7 +620,6 @@ do_import(JSContext       *context,
 
     g_free(full_path);
     g_free(filename);
-    g_free(dirname);
 
     if (!result &&
         !JS_IsExceptionPending(context)) {
@@ -697,11 +639,13 @@ do_import(JSContext       *context,
 static bool
 importer_enumerate(JSContext        *context,
                    JS::HandleObject  object,
-                   JS::AutoIdVector& properties)
+                   JS::AutoIdVector& properties,
+                   bool              enumerable_only)
 {
     Importer *priv;
     guint32 search_path_len;
     guint32 i;
+    bool is_array;
 
     priv = priv_from_js(context, object);
 
@@ -715,7 +659,9 @@ importer_enumerate(JSContext        *context,
                                      &search_path))
         return false;
 
-    if (!JS_IsArrayObject(context, search_path)) {
+    if (!JS_IsArrayObject(context, search_path, &is_array))
+        return false;
+    if (!is_array) {
         gjs_throw(context, "searchPath property on importer is not an array");
         return false;
     }
@@ -727,7 +673,7 @@ importer_enumerate(JSContext        *context,
 
     JS::RootedValue elem(context);
     for (i = 0; i < search_path_len; ++i) {
-        char *dirname = NULL;
+        GjsAutoJSChar dirname(context);
         char *init_path;
 
         elem.setUndefined();
@@ -757,24 +703,19 @@ importer_enumerate(JSContext        *context,
 
         /* new_for_commandline_arg handles resource:/// paths */
         GjsAutoUnref<GFile> dir = g_file_new_for_commandline_arg(dirname);
-        g_free(dirname);
         GjsAutoUnref<GFileEnumerator> direnum =
             g_file_enumerate_children(dir, G_FILE_ATTRIBUTE_STANDARD_TYPE,
                                       G_FILE_QUERY_INFO_NONE, NULL, NULL);
 
         while (true) {
             GFileInfo *info;
-            GError *error = NULL;
-            info = g_file_enumerator_next_file(direnum, NULL, &error);
-            if (error != NULL) {
-                g_error_free (error);
+            GFile *file;
+            if (!g_file_enumerator_iterate(direnum, &info, &file, NULL, NULL))
                 break;
-            }
-            if (info == NULL)
+            if (info == NULL || file == NULL)
                 break;
 
-            GjsAutoChar filename =
-              g_path_get_basename(g_file_info_get_name(info));
+            GjsAutoChar filename = g_file_get_basename(file);
 
             /* skip hidden files and directories (.svn, .git, ...) */
             if (filename[0] == '.')
@@ -785,12 +726,14 @@ importer_enumerate(JSContext        *context,
                 continue;
 
             if (g_file_info_get_file_type(info) == G_FILE_TYPE_DIRECTORY) {
-                properties.append(gjs_intern_string_to_id(context, filename));
+                if (!properties.append(gjs_intern_string_to_id(context, filename)))
+                    g_error("Unable to append to vector");
             } else if (g_str_has_suffix(filename, "." G_MODULE_SUFFIX) ||
                        g_str_has_suffix(filename, ".js")) {
                 GjsAutoChar filename_noext =
                     g_strndup(filename, strlen(filename) - 3);
-                properties.append(gjs_intern_string_to_id(context, filename_noext));
+                if (!properties.append(gjs_intern_string_to_id(context, filename_noext)))
+                    g_error("Unable to append to vector");
             }
         }
     }
@@ -809,8 +752,8 @@ importer_resolve(JSContext        *context,
                  bool             *resolved)
 {
     Importer *priv;
-    char *name = NULL;
     jsid module_init_name;
+    GjsAutoJSChar name(context);
 
     module_init_name = gjs_context_get_const_string(context, GJS_STRING_MODULE_INIT);
     if (id == module_init_name) {
@@ -826,103 +769,29 @@ importer_resolve(JSContext        *context,
         strcmp(name, "toString") == 0 ||
         strcmp(name, "__iterator__") == 0) {
         *resolved = false;
-        g_free(name);
         return true;
     }
     priv = priv_from_js(context, obj);
 
     gjs_debug_jsprop(GJS_DEBUG_IMPORTER,
                      "Resolve prop '%s' hook obj %p priv %p",
-                     name, obj.get(), priv);
-    if (priv == NULL) {
+                     name.get(), obj.get(), priv);
+    if (!priv) {
         /* we are the prototype, or have the wrong class */
         *resolved = false;
-        g_free(name);
         return true;
     }
 
     JSAutoRequest ar(context);
-    if (!do_import(context, obj, priv, name)) {
-        g_free(name);
+    if (!do_import(context, obj, priv, id, name))
         return false;
-    }
 
     *resolved = true;
-    g_free(name);
     return true;
 }
 
 GJS_NATIVE_CONSTRUCTOR_DEFINE_ABSTRACT(importer)
 
-static bool
-gjs_importer_add_subimporter(JSContext *context,
-                             unsigned   argc,
-                             JS::Value *vp)
-{
-    if (argc != 2) {
-        gjs_throw(context, "Must pass two arguments to addSubImporter()");
-        return false;
-    }
-
-    GJS_GET_THIS(context, argc, vp, argv, importer);
-
-    bool success = false;
-
-    JSExceptionState *exc_state;
-
-    char *name;
-    char *path;
-    char *search_path[2] = { 0, 0 };
-
-    JS_BeginRequest(context);
-
-    /* JS::ToString might throw, in which case we will only log that the value
-     * could not be converted to string */
-    exc_state = JS_SaveExceptionState(context);
-    JS::RootedString name_str(context, JS::ToString(context, argv[0]));
-    if (name_str != NULL)
-        argv[0].setString(name_str);
-    JS_RestoreExceptionState(context, exc_state);
-
-    if (name_str == NULL) {
-        g_message("addSubImporter: <cannot convert name value to string>");
-        JS_EndRequest(context);
-        return false;
-    }
-
-    exc_state = JS_SaveExceptionState(context);
-    JS::RootedString path_str(context, JS::ToString(context, argv[1]));
-    if (path_str != NULL)
-        argv[1].setString(path_str);
-    JS_RestoreExceptionState(context, exc_state);
-
-    if (path_str == NULL) {
-        g_message("addSubImporter: <cannot convert path value to string>");
-        JS_EndRequest(context);
-        return false;
-    }
-
-    if (!gjs_string_to_utf8(context, JS::StringValue(name_str), &name))
-        goto out;
-
-    if (!gjs_string_to_utf8(context, JS::StringValue(path_str), &path))
-        goto out;
-
-    search_path[0] = path;
-
-    gjs_define_importer(context, importer, name, (const char **)search_path, FALSE);
-
-    JS_EndRequest(context);
-    argv.rval().setUndefined();
-    success = true;
-
-    out:
-
-    g_free (name);
-    g_free (path);
-    return success;
-}
-
 static void
 importer_finalize(js::FreeOp *fop,
                   JSObject   *obj)
@@ -932,7 +801,7 @@ importer_finalize(js::FreeOp *fop,
     priv = (Importer*) JS_GetPrivate(obj);
     gjs_debug_lifecycle(GJS_DEBUG_IMPORTER,
                         "finalize, obj %p priv %p", obj, priv);
-    if (priv == NULL)
+    if (!priv)
         return; /* we are the prototype, not a real instance */
 
     GJS_DEC_COUNTER(importer);
@@ -943,38 +812,38 @@ importer_finalize(js::FreeOp *fop,
  * instances of the object, and to the prototype that instances of the
  * class have.
  */
-const js::Class gjs_importer_real_class = {
-    "GjsFileImporter",
-    JSCLASS_HAS_PRIVATE |
-    JSCLASS_IMPLEMENTS_BARRIERS,
+static const js::ClassOps gjs_importer_class_ops = {
     NULL,  /* addProperty */
     NULL,  /* deleteProperty */
     NULL,  /* getProperty */
     NULL,  /* setProperty */
     NULL,  /* enumerate (see below) */
     importer_resolve,
-    NULL,  /* convert */
-    importer_finalize,
-    NULL,  /* call */
-    NULL,  /* hasInstance */
-    NULL,  /* construct */
-    NULL,  /* trace */
-    JS_NULL_CLASS_SPEC,
-    JS_NULL_CLASS_EXT,
-    {
-        NULL,  /* lookupProperty */
-        NULL,  /* defineProperty */
-        NULL,  /* hasProperty */
-        NULL,  /* getProperty */
-        NULL,  /* setProperty */
-        NULL,  /* getOwnPropertyDescriptor */
-        NULL,  /* deleteProperty */
-        NULL,  /* watch */
-        NULL,  /* unwatch */
-        NULL,  /* getElements */
-        importer_enumerate,
-        NULL,  /* thisObject */
-    }
+    nullptr,  /* mayResolve */
+    importer_finalize
+};
+
+static const js::ObjectOps gjs_importer_object_ops = {
+    NULL,  /* lookupProperty */
+    NULL,  /* defineProperty */
+    NULL,  /* hasProperty */
+    NULL,  /* getProperty */
+    NULL,  /* setProperty */
+    NULL,  /* getOwnPropertyDescriptor */
+    NULL,  /* deleteProperty */
+    NULL,  /* watch */
+    NULL,  /* unwatch */
+    NULL,  /* getElements */
+    importer_enumerate
+};
+
+const js::Class gjs_importer_real_class = {
+    "GjsFileImporter",
+    JSCLASS_HAS_PRIVATE | JSCLASS_FOREGROUND_FINALIZE,
+    &gjs_importer_class_ops,
+    nullptr,
+    nullptr,
+    &gjs_importer_object_ops
 };
 
 static JSPropertySpec *gjs_importer_proto_props = nullptr;
@@ -987,11 +856,6 @@ JSFunctionSpec gjs_importer_proto_funcs[] = {
 
 GJS_DEFINE_PROTO_FUNCS(importer)
 
-JSFunctionSpec gjs_global_importer_funcs[] = {
-    JS_FS("addSubImporter", gjs_importer_add_subimporter, 0, 0),
-    JS_FS_END
-};
-
 static JSObject*
 importer_new(JSContext *context,
              bool       is_root)
@@ -999,12 +863,12 @@ importer_new(JSContext *context,
     Importer *priv;
 
     JS::RootedObject proto(context);
-    if (!gjs_importer_define_proto(context, JS::NullPtr(), &proto))
+    if (!gjs_importer_define_proto(context, nullptr, &proto))
         g_error("Error creating importer prototype");
 
     JS::RootedObject importer(context,
         JS_NewObjectWithGivenProto(context, &gjs_importer_class, proto));
-    if (importer == NULL)
+    if (!importer)
         g_error("No memory to create importer");
 
     priv = g_slice_new0(Importer);
@@ -1085,12 +949,12 @@ gjs_get_search_path(void)
 }
 
 static JSObject*
-gjs_create_importer(JSContext       *context,
-                    const char      *importer_name,
-                    const char     **initial_search_path,
-                    bool             add_standard_search_path,
-                    bool             is_root,
-                    JS::HandleObject in_object)
+gjs_create_importer(JSContext          *context,
+                    const char         *importer_name,
+                    const char * const *initial_search_path,
+                    bool                add_standard_search_path,
+                    bool                is_root,
+                    JS::HandleObject    in_object)
 {
     char **paths[2] = {0};
     char **search_path;
@@ -1109,7 +973,7 @@ gjs_create_importer(JSContext       *context,
     if (!gjs_define_string_array(context, importer,
                                  "searchPath", -1, (const char **)search_path,
                                  /* settable (no READONLY) but not deleteable (PERMANENT) */
-                                 JSPROP_PERMANENT))
+                                 JSPROP_PERMANENT | JSPROP_RESOLVING))
         g_error("no memory to define importer search path prop");
 
     g_strfreev(search_path);
@@ -1120,7 +984,7 @@ gjs_create_importer(JSContext       *context,
     return importer;
 }
 
-JSObject*
+static JSObject *
 gjs_define_importer(JSContext       *context,
                     JS::HandleObject in_object,
                     const char      *importer_name,
@@ -1143,69 +1007,9 @@ gjs_define_importer(JSContext       *context,
     return importer;
 }
 
-/* If this were called twice for the same runtime with different args it
- * would basically be a bug, but checking for that is a lot of code so
- * we just ignore all calls after the first and hope the args are the same.
- */
-bool
-gjs_create_root_importer(JSContext   *context,
-                         const char **initial_search_path,
-                         bool         add_standard_search_path)
+JSObject *
+gjs_create_root_importer(JSContext          *cx,
+                         const char * const *search_path)
 {
-    JS::Value importer_val;
-    JSObject *importer;
-
-    JS_BeginRequest(context);
-
-    importer_val = gjs_get_global_slot(context, GJS_GLOBAL_SLOT_IMPORTS);
-
-    if (G_UNLIKELY (!importer_val.isUndefined())) {
-        gjs_debug(GJS_DEBUG_IMPORTER,
-                  "Someone else already created root importer, ignoring second request");
-
-        JS_EndRequest(context);
-        return true;
-    }
-
-    importer = gjs_create_importer(context, "imports",
-                                   initial_search_path,
-                                   add_standard_search_path,
-                                   true, JS::NullPtr());
-
-    JS::RootedObject global(context, importer);
-
-    importer_val = JS::ObjectValue (*importer);
-
-    JS_DefineFunctions(context, global, &gjs_global_importer_funcs[0]);
-    gjs_set_global_slot(context, GJS_GLOBAL_SLOT_IMPORTS, importer_val);
-
-    JS_EndRequest(context);
-    return true;
-}
-
-bool
-gjs_define_root_importer_object(JSContext        *context,
-                                JS::HandleObject  in_object,
-                                JS::HandleObject  root_importer)
-{
-    JSAutoRequest ar(context);
-
-    if (!gjs_object_define_property(context, in_object,
-                                    GJS_STRING_IMPORTS, root_importer,
-                                    GJS_MODULE_PROP_FLAGS)) {
-        gjs_debug(GJS_DEBUG_IMPORTER, "DefineProperty imports on %p failed",
-                  in_object.get());
-        return false;
-    }
-
-    return true;
-}
-
-bool
-gjs_define_root_importer(JSContext       *cx,
-                         JS::HandleObject in_object)
-{
-    JS::Value importer = gjs_get_global_slot(cx, GJS_GLOBAL_SLOT_IMPORTS);
-    JS::RootedObject rooted_importer(cx, &importer.toObject());
-    return gjs_define_root_importer_object(cx, in_object, rooted_importer);
-}
+    return gjs_create_importer(cx, "imports", search_path, true, true, nullptr);
+}
\ No newline at end of file
