@@ -45,101 +45,6 @@ gjs_util_error_quark (void)
     return g_quark_from_static_string ("gjs-util-error-quark");
 }
 
-static JSClass global_class = {
-    "GjsGlobal",
-    JSCLASS_GLOBAL_FLAGS_WITH_SLOTS(GJS_GLOBAL_SLOT_LAST) |
-    JSCLASS_IMPLEMENTS_BARRIERS,
-    NULL,  /* addProperty */
-    NULL,  /* deleteProperty */
-    NULL,  /* getProperty */
-    NULL,  /* setProperty */
-    NULL,  /* enumerate */
-    NULL,  /* resolve */
-    NULL,  /* convert */
-    NULL,  /* finalize */
-    NULL,  /* call */
-    NULL,  /* hasInstance */
-    NULL,  /* construct */
-    JS_GlobalObjectTraceHook
-};
-
-/**
- * gjs_init_context_standard:
- * @context: a #JSContext
- * @global_out: (out): The created global object.
-
- * This function creates a global object given the context,
- * and initializes it with the default API.
- *
- * Returns: true on success, false otherwise
- */
-bool
-gjs_init_context_standard (JSContext              *context,
-                           JS::MutableHandleObject global)
-{
-    JS::CompartmentOptions compartment_options;
-
-    bool extra_warnings = false;
-    if (!g_getenv("GJS_DISABLE_EXTRA_WARNINGS")) {
-        gjs_debug(GJS_DEBUG_CONTEXT, "Enabling extra warnings");
-        extra_warnings = true;
-    }
-
-    /* setDontReportUncaught: Don't send exceptions to our error report handler;
-     * instead leave them set. This allows us to get at the exception object.
-     *
-     * setExtraWarnings: Report warnings to error reporter function.
-     */
-    JS::ContextOptionsRef(context).setDontReportUncaught(true);
-    JS::RuntimeOptionsRef(context).setExtraWarnings(extra_warnings);
-
-    if (!g_getenv("GJS_DISABLE_JIT")) {
-        gjs_debug(GJS_DEBUG_CONTEXT, "Enabling JIT");
-        JS::RuntimeOptionsRef(context)
-            .setIon(true)
-            .setBaseline(true)
-            .setAsmJS(true);
-    }
-
-    compartment_options.setVersion(JSVERSION_LATEST);
-    global.set(JS_NewGlobalObject(context, &global_class, NULL,
-                                  JS::FireOnNewGlobalHook, compartment_options));
-    if (global == NULL)
-        return false;
-
-    JSAutoCompartment ac(context, global);
-
-    if (!JS_InitStandardClasses(context, global))
-        return false;
-
-    if (!JS_InitReflect(context, global))
-        return false;
-
-    if (!JS_DefineDebuggerObject(context, global))
-        return false;
-
-    return true;
-}
-
-void
-gjs_set_global_slot (JSContext     *context,
-                     GjsGlobalSlot  slot,
-                     JS::Value      value)
-{
-    JSObject *global;
-    global = gjs_get_import_global(context);
-    JS_SetReservedSlot(global, JSCLASS_GLOBAL_SLOT_COUNT + slot, value);
-}
-
-JS::Value
-gjs_get_global_slot (JSContext     *context,
-                     GjsGlobalSlot  slot)
-{
-    JSObject *global;
-    global = gjs_get_import_global(context);
-    return JS_GetReservedSlot(global, JSCLASS_GLOBAL_SLOT_COUNT + slot);
-}
-
 bool
 gjs_object_get_property(JSContext             *cx,
                         JS::HandleObject       obj,
@@ -201,6 +106,34 @@ gjs_object_define_property(JSContext       *cx,
                                  value, flags, getter, setter);
 }
 
+bool
+gjs_object_define_property(JSContext       *cx,
+                           JS::HandleObject obj,
+                           GjsConstString   property_name,
+                           JS::HandleString value,
+                           unsigned         flags,
+                           JSNative         getter,
+                           JSNative         setter)
+{
+    return JS_DefinePropertyById(cx, obj,
+                                 gjs_context_get_const_string(cx, property_name),
+                                 value, flags, getter, setter);
+}
+
+bool
+gjs_object_define_property(JSContext       *cx,
+                           JS::HandleObject obj,
+                           GjsConstString   property_name,
+                           uint32_t         value,
+                           unsigned         flags,
+                           JSNative         getter,
+                           JSNative         setter)
+{
+    return JS_DefinePropertyById(cx, obj,
+                                 gjs_context_get_const_string(cx, property_name),
+                                 value, flags, getter, setter);
+}
+
 static void
 throw_property_lookup_error(JSContext       *cx,
                             JS::HandleObject obj,
@@ -211,17 +144,15 @@ throw_property_lookup_error(JSContext       *cx,
     /* remember gjs_throw() is a no-op if JS_GetProperty()
      * already set an exception
      */
-    char *name;
+    GjsAutoJSChar name(cx);
     gjs_get_string_id(cx, property_name, &name);
 
     if (description)
-        gjs_throw(cx, "No property '%s' in %s (or %s)", name, description,
+        gjs_throw(cx, "No property '%s' in %s (or %s)", name.get(), description,
                   reason);
     else
-        gjs_throw(cx, "No property '%s' in object %p (or %s)", name,
+        gjs_throw(cx, "No property '%s' in object %p (or %s)", name.get(),
                   obj.address(), reason);
-
-    g_free(name);
 }
 
 /* Returns whether the object had the property; if the object did
@@ -299,13 +230,12 @@ gjs_object_require_property(JSContext       *cx,
                             JS::HandleObject obj,
                             const char      *description,
                             JS::HandleId     property_name,
-                            char           **value)
+                            GjsAutoJSChar   *value)
 {
     JS::RootedValue prop_value(cx);
     if (JS_GetPropertyById(cx, obj, property_name, &prop_value) &&
-        gjs_string_to_utf8(cx, prop_value, value)) {
+        gjs_string_to_utf8(cx, prop_value, value))
         return true;
-    }
 
     throw_property_lookup_error(cx, obj, description, property_name,
                                 "it was not a valid string");
@@ -385,12 +315,14 @@ gjs_build_string_array(JSContext   *context,
         array_length = g_strv_length(array_values);
 
     JS::AutoValueVector elems(context);
-    elems.reserve(array_length);
+    if (!elems.reserve(array_length))
+        g_error("Unable to reserve memory for vector");
 
     for (i = 0; i < array_length; ++i) {
         JS::RootedValue element(context,
             JS::StringValue(JS_NewStringCopyZ(context, array_values[i])));
-        elems.append(element);
+        if (!elems.append(element))
+            g_error("Unable to append to vector");
     }
 
     return JS_NewArrayObject(context, elems);
@@ -409,11 +341,11 @@ gjs_define_string_array(JSContext       *context,
     JS::RootedObject array(context,
         gjs_build_string_array(context, array_length, (char **) array_values));
 
-    if (array == NULL)
-        return NULL;
+    if (!array)
+        return nullptr;
 
     if (!JS_DefineProperty(context, in_object, array_name, array, attrs))
-        return NULL;
+        return nullptr;
 
     return array;
 }
@@ -432,7 +364,7 @@ gjs_string_readable (JSContext   *context,
                      JSString    *string)
 {
     GString *buf = g_string_new("");
-    char *chars;
+    GjsAutoJSChar chars(context);
 
     JS_BeginRequest(context);
 
@@ -454,7 +386,6 @@ gjs_string_readable (JSContext   *context,
         g_free(escaped);
     } else {
         g_string_append(buf, chars);
-        g_free(chars);
     }
 
     g_string_append_c(buf, '"');
@@ -473,7 +404,7 @@ _gjs_g_utf8_make_valid (const char *name)
 
     g_return_val_if_fail (name != NULL, NULL);
 
-    string = NULL;
+    string = nullptr;
     remainder = name;
     remaining_bytes = strlen (name);
 
@@ -482,7 +413,7 @@ _gjs_g_utf8_make_valid (const char *name)
             break;
         valid_bytes = invalid - remainder;
 
-        if (string == NULL)
+        if (!string)
             string = g_string_sized_new (remaining_bytes);
 
         g_string_append_len (string, remainder, valid_bytes);
@@ -493,7 +424,7 @@ _gjs_g_utf8_make_valid (const char *name)
         remainder = invalid + 1;
     }
 
-    if (string == NULL)
+    if (!string)
         return g_strdup (name);
 
     g_string_append (string, remainder);
@@ -526,7 +457,7 @@ gjs_value_debug_string(JSContext      *context,
 
     JS::RootedString str(context, JS::ToString(context, value));
 
-    if (str == NULL) {
+    if (!str) {
         if (value.isObject()) {
             /* Specifically the Call object (see jsfun.c in spidermonkey)
              * does not have a toString; there may be others also.
@@ -535,7 +466,7 @@ gjs_value_debug_string(JSContext      *context,
             if (klass != NULL) {
                 str = JS_NewStringCopyZ(context, klass->name);
                 JS_ClearPendingException(context);
-                if (str == NULL) {
+                if (!str) {
                     JS_EndRequest(context);
                     return g_strdup("[out of memory copying class name]");
                 }
@@ -550,7 +481,7 @@ gjs_value_debug_string(JSContext      *context,
         }
     }
 
-    g_assert(str != NULL);
+    g_assert(str);
 
     bytes = JS_EncodeStringToUTF8(context, str);
     JS_EndRequest(context);
@@ -565,11 +496,11 @@ static char *
 utf8_exception_from_non_gerror_value(JSContext      *cx,
                                      JS::HandleValue exc)
 {
-    char *utf8_exception = NULL;
+    GjsAutoJSChar utf8_exception(cx);
     JS::RootedString exc_str(cx, JS::ToString(cx, exc));
-    if (exc_str != NULL)
+    if (exc_str)
         gjs_string_to_utf8(cx, JS::StringValue(exc_str), &utf8_exception);
-    return utf8_exception;
+    return utf8_exception.copy();
 }
 
 bool
@@ -577,7 +508,8 @@ gjs_log_exception_full(JSContext       *context,
                        JS::HandleValue  exc,
                        JS::HandleString message)
 {
-    char *utf8_exception, *utf8_message;
+    char *utf8_exception;
+    GjsAutoJSChar utf8_message(context);
     bool is_syntax;
 
     JS_BeginRequest(context);
@@ -596,7 +528,7 @@ gjs_log_exception_full(JSContext       *context,
                                              gerror->message);
         } else {
             JS::RootedValue js_name(context);
-            char *utf8_name;
+            GjsAutoJSChar utf8_name(context);
 
             if (gjs_object_get_property(context, exc_obj,
                                         GJS_STRING_NAME, &js_name) &&
@@ -609,10 +541,8 @@ gjs_log_exception_full(JSContext       *context,
         }
     }
 
-    if (message != NULL)
+    if (message)
         gjs_string_to_utf8(context, JS::StringValue(message), &utf8_message);
-    else
-        utf8_message = NULL;
 
     /* We log syntax errors differently, because the stack for those includes
        only the referencing module, but we want to print out the filename and
@@ -622,7 +552,7 @@ gjs_log_exception_full(JSContext       *context,
     if (is_syntax) {
         JS::RootedValue js_lineNumber(context), js_fileName(context);
         unsigned lineNumber;
-        char *utf8_fileName;
+        GjsAutoJSChar utf8_filename(context);
 
         gjs_object_get_property(context, exc_obj, GJS_STRING_LINE_NUMBER,
                                 &js_lineNumber);
@@ -630,50 +560,47 @@ gjs_log_exception_full(JSContext       *context,
                                 &js_fileName);
 
         if (js_fileName.isString())
-            gjs_string_to_utf8(context, js_fileName, &utf8_fileName);
+            gjs_string_to_utf8(context, js_fileName, &utf8_filename);
         else
-            utf8_fileName = g_strdup("unknown");
+            utf8_filename.reset(context, JS_strdup(context, "unknown"));
 
         lineNumber = js_lineNumber.toInt32();
 
-        if (utf8_message) {
-            g_critical("JS ERROR: %s: %s @ %s:%u", utf8_message, utf8_exception,
-                       utf8_fileName, lineNumber);
+        if (message) {
+            g_critical("JS ERROR: %s: %s @ %s:%u", utf8_message.get(), utf8_exception,
+                       utf8_filename.get(), lineNumber);
         } else {
             g_critical("JS ERROR: %s @ %s:%u", utf8_exception,
-                       utf8_fileName, lineNumber);
+                       utf8_filename.get(), lineNumber);
         }
 
-        g_free(utf8_fileName);
     } else {
-        char *utf8_stack;
+        GjsAutoJSChar utf8_stack(context);
         JS::RootedValue stack(context);
+        bool have_utf8_stack = false;
 
         if (exc.isObject() &&
             gjs_object_get_property(context, exc_obj, GJS_STRING_STACK,
                                     &stack) &&
-            stack.isString())
+            stack.isString()) {
             gjs_string_to_utf8(context, stack, &utf8_stack);
-        else
-            utf8_stack = NULL;
+            have_utf8_stack = true;
+        }
 
-        if (utf8_message) {
-            if (utf8_stack)
-                g_warning("JS ERROR: %s: %s\n%s", utf8_message, utf8_exception, utf8_stack);
+        if (message) {
+            if (have_utf8_stack)
+                g_warning("JS ERROR: %s: %s\n%s", utf8_message.get(), utf8_exception, utf8_stack.get());
             else
-                g_warning("JS ERROR: %s: %s", utf8_message, utf8_exception);
+                g_warning("JS ERROR: %s: %s", utf8_message.get(), utf8_exception);
         } else {
-            if (utf8_stack)
-                g_warning("JS ERROR: %s\n%s", utf8_exception, utf8_stack);
+            if (have_utf8_stack)
+                g_warning("JS ERROR: %s\n%s", utf8_exception, utf8_stack.get());
             else
                 g_warning("JS ERROR: %s", utf8_exception);
         }
-
-        g_free(utf8_stack);
     }
 
     g_free(utf8_exception);
-    g_free(utf8_message);
 
     JS_EndRequest(context);
 
@@ -693,7 +620,7 @@ gjs_log_exception(JSContext  *context)
 
     JS_ClearPendingException(context);
 
-    gjs_log_exception_full(context, exc, JS::NullPtr());
+    gjs_log_exception_full(context, exc, nullptr);
 
     retval = true;
 
@@ -779,7 +706,7 @@ _linux_get_self_process_size (gulong *vm_size,
 }
 
 static gulong linux_rss_trigger;
-static gint64 last_gc_time;
+static int64_t last_gc_check_time;
 #endif
 
 void
@@ -795,9 +722,11 @@ gjs_gc_if_needed (JSContext *context)
         /* We rate limit GCs to at most one per 5 frames.
            One frame is 16666 microseconds (1000000/60)*/
         now = g_get_monotonic_time();
-        if (now - last_gc_time < 5 * 16666)
+        if (now - last_gc_check_time < 5 * 16666)
             return;
 
+        last_gc_check_time = now;
+
         _linux_get_self_process_size (&vmsize, &rss_size);
 
         /* linux_rss_trigger is initialized to 0, so currently
@@ -812,8 +741,7 @@ gjs_gc_if_needed (JSContext *context)
          */
         if (rss_size > linux_rss_trigger) {
             linux_rss_trigger = (gulong) MIN(G_MAXULONG, rss_size * 1.25);
-            JS_GC(JS_GetRuntime(context));
-            last_gc_time = now;
+            JS_GC(context);
         } else if (rss_size < (0.75 * linux_rss_trigger)) {
             /* If we've shrunk by 75%, lower the trigger */
             linux_rss_trigger = (rss_size * 1.25);
@@ -938,11 +866,13 @@ gjs_eval_with_scope(JSContext             *context,
            .setSourceIsLazy(true);
 
     JS::RootedScript compiled_script(context);
-    if (!JS::Compile(context, object, options, script, real_len, &compiled_script))
+    if (!JS::Compile(context, options, script, real_len, &compiled_script))
         return false;
 
     JS::AutoObjectVector scope_chain(context);
-    scope_chain.append(eval_obj);
+    if (!scope_chain.append(eval_obj))
+        g_error("Unable to append to vector");
+
     if (!JS_ExecuteScript(context, scope_chain, compiled_script, retval))
         return false;
 
