@@ -173,7 +173,6 @@ gjs_callback_closure(ffi_cif *cif,
                      void *data)
 {
     JSContext *context;
-    JSRuntime *runtime;
     JSObject *func_obj;
     GjsCallbackTrampoline *trampoline;
     int i, n_args, n_jsargs, n_outargs;
@@ -186,18 +185,19 @@ gjs_callback_closure(ffi_cif *cif,
     gjs_callback_trampoline_ref(trampoline);
 
     context = trampoline->context;
-    runtime = JS_GetRuntime(context);
-    if (G_UNLIKELY (gjs_runtime_is_sweeping(runtime))) {
+    if (G_UNLIKELY(_gjs_context_is_sweeping(context))) {
         g_critical("Attempting to call back into JSAPI during the sweeping phase of GC. "
                    "This is most likely caused by not destroying a Clutter actor or Gtk+ "
                    "widget with ::destroy signals connected, but can also be caused by "
-                   "using the destroy() or dispose() vfuncs. Because it would crash the "
-                   "application, it has been blocked and the JS callback not invoked.");
-        /* A gjs_dumpstack() would be nice here, but we can't,
-           because that works by creating a new Error object and
-           reading the stack property, which is the worst possible
-           idea during a GC session.
-        */
+                   "using the destroy(), dispose(), or remove() vfuncs. "
+                   "Because it would crash the application, it has been "
+                   "blocked and the JS callback not invoked.");
+        if (trampoline->info) {
+            const char *name = g_base_info_get_name(static_cast<GIBaseInfo *>(trampoline->info));
+            g_critical("The offending callback was %s()%s.", name,
+                       trampoline->is_vfunc ? ", a vfunc" : "");
+        }
+        gjs_dumpstack();
         gjs_callback_trampoline_unref(trampoline);
         return;
     }
@@ -212,7 +212,10 @@ gjs_callback_closure(ffi_cif *cif,
 
     n_outargs = 0;
     JS::AutoValueVector jsargs(context);
-    jsargs.reserve(n_args);
+
+    if (!jsargs.reserve(n_args))
+        g_error("Unable to reserve space for vector");
+
     JS::RootedValue rval(context);
     JS::RootedValue rooted_function(context, trampoline->js_function);
     JS::RootedObject this_object(context);
@@ -255,14 +258,18 @@ gjs_callback_closure(ffi_cif *cif,
                                                (GArgument *) args[array_length_pos], true))
                     goto out;
 
-                jsargs.growBy(1);
+                if (!jsargs.growBy(1))
+                    g_error("Unable to grow vector");
+
                 if (!gjs_value_from_explicit_array(context, jsargs[n_jsargs++],
                                                    &type_info, (GArgument*) args[i], length.toInt32()))
                     goto out;
                 break;
             }
             case PARAM_NORMAL:
-                jsargs.growBy(1);
+                if (!jsargs.growBy(1))
+                    g_error("Unable to grow vector");
+
                 if (!gjs_value_from_g_argument(context, jsargs[n_jsargs++],
                                                &type_info,
                                                (GArgument *) args[i], false))
@@ -769,9 +776,9 @@ gjs_invoke_c_function(JSContext                             *context,
      */
     if (args.length() > function->expected_js_argc) {
         GjsAutoChar name = format_function_name(function, is_method);
-        JS_ReportWarning(context, "Too many arguments to %s: expected %d, "
-                         "got %" G_GSIZE_FORMAT, name.get(),
-                         function->expected_js_argc, args.length());
+        JS_ReportWarningUTF8(context, "Too many arguments to %s: expected %d, "
+                             "got %" G_GSIZE_FORMAT, name.get(),
+                             function->expected_js_argc, args.length());
     } else if (args.length() < function->expected_js_argc) {
         GjsAutoChar name = format_function_name(function, is_method);
         gjs_throw(context, "Too few arguments to %s: "
@@ -1040,7 +1047,8 @@ gjs_invoke_c_function(JSContext                             *context,
     /* Only process return values if the function didn't throw */
     if (function->js_out_argc > 0 && !did_throw_gerror) {
         for (size_t i = 0; i < function->js_out_argc; i++)
-            return_values.append(JS::UndefinedValue());
+            if (!return_values.append(JS::UndefinedValue()))
+                g_error("Unable to append to vector");
 
         if (return_tag != GI_TYPE_TAG_VOID) {
             GITransfer transfer = g_callable_info_get_caller_owns((GICallableInfo*) function->info);
@@ -1483,22 +1491,24 @@ function_to_string (JSContext *context,
  * instances of the object, and to the prototype that instances of the
  * class have.
  */
-struct JSClass gjs_function_class = {
-    "GIRepositoryFunction", /* means "new GIRepositoryFunction()" works */
-    JSCLASS_HAS_PRIVATE |
-    JSCLASS_BACKGROUND_FINALIZE |
-    JSCLASS_IMPLEMENTS_BARRIERS,
+static const struct JSClassOps gjs_function_class_ops = {
     NULL,  /* addProperty */
     NULL,  /* deleteProperty */
     NULL,  /* getProperty */
     NULL,  /* setProperty */
     NULL,  /* enumerate */
     NULL,  /* resolve */
-    NULL,  /* convert */
+    nullptr,  /* mayResolve */
     function_finalize,
     function_call
 };
 
+struct JSClass gjs_function_class = {
+    "GIRepositoryFunction", /* means "new GIRepositoryFunction()" works */
+    JSCLASS_HAS_PRIVATE | JSCLASS_BACKGROUND_FINALIZE,
+    &gjs_function_class_ops
+};
+
 static JSPropertySpec gjs_function_proto_props[] = {
     JS_PSG("length", get_num_arguments, JSPROP_PERMANENT),
     JS_PS_END
@@ -1678,12 +1688,12 @@ function_new(JSContext      *context,
     Function *priv;
 
     JS::RootedObject proto(context);
-    if (!gjs_function_define_proto(context, JS::NullPtr(), &proto))
+    if (!gjs_function_define_proto(context, nullptr, &proto))
         return nullptr;
 
     JS::RootedObject function(context,
         JS_NewObjectWithGivenProto(context, &gjs_function_class, proto));
-    if (function == NULL) {
+    if (!function) {
         gjs_debug(GJS_DEBUG_GFUNCTION, "Failed to construct function");
         return NULL;
     }
@@ -1720,7 +1730,7 @@ gjs_define_function(JSContext       *context,
     JSAutoRequest ar(context);
 
     JS::RootedObject function(context, function_new(context, gtype, info));
-    if (function == NULL)
+    if (!function)
         return NULL;
 
     if (info_type == GI_INFO_TYPE_FUNCTION) {
