@@ -241,7 +241,7 @@ fundamental_instance_resolve_interface(JSContext       *context,
                                        JS::HandleObject obj,
                                        bool            *resolved,
                                        Fundamental     *proto_priv,
-                                       char            *name)
+                                       const char      *name)
 {
     GIFunctionInfo *method_info;
     bool ret;
@@ -301,7 +301,7 @@ fundamental_instance_resolve(JSContext       *context,
                              bool            *resolved)
 {
     FundamentalInstance *priv;
-    char *name = NULL;
+    GjsAutoJSChar name(context);
 
     if (!gjs_get_string_id(context, id, &name)) {
         *resolved = false;
@@ -311,12 +311,10 @@ fundamental_instance_resolve(JSContext       *context,
     priv = priv_from_js(context, obj);
     gjs_debug_jsprop(GJS_DEBUG_GFUNDAMENTAL,
                      "Resolve prop '%s' hook obj %p priv %p",
-                     name, obj.get(), priv);
+                     name.get(), obj.get(), priv);
 
-    if (priv == NULL) {
-        g_free(name);
+    if (priv == nullptr)
         return false; /* wrong class */
-    }
 
     if (!fundamental_is_prototype(priv)) {
         /* We are an instance, not a prototype, so look for
@@ -327,7 +325,6 @@ fundamental_instance_resolve(JSContext       *context,
          * hooks, not this resolve hook.
          */
         *resolved = false;
-        g_free(name);
         return true;
     }
 
@@ -356,7 +353,6 @@ fundamental_instance_resolve(JSContext       *context,
                           g_base_info_get_name((GIBaseInfo *) proto_priv->info));
                 g_base_info_unref((GIBaseInfo *) method_info);
                 *resolved = false;
-                g_free(name);
                 return true;
             }
 
@@ -369,7 +365,6 @@ fundamental_instance_resolve(JSContext       *context,
             if (gjs_define_function(context, obj, proto_priv->gtype,
                                     method_info) == NULL) {
                 g_base_info_unref((GIBaseInfo *) method_info);
-                g_free(name);
                 return false;
             }
 
@@ -384,7 +379,6 @@ fundamental_instance_resolve(JSContext       *context,
     bool status =
         fundamental_instance_resolve_interface(context, obj, resolved,
                                                proto_priv, name);
-    g_free(name);
     return status;
 }
 
@@ -537,8 +531,8 @@ fundamental_trace(JSTracer *tracer,
     if (priv == nullptr || !fundamental_is_prototype(priv))
         return;  /* Only prototypes need tracing */
 
-    JS_CallIdTracer(tracer, &priv->constructor_name,
-                    "Fundamental::constructor_name");
+    JS::TraceEdge<jsid>(tracer, &priv->constructor_name,
+                        "Fundamental::constructor_name");
 }
 
 /* The bizarre thing about this vtable is that it applies to both
@@ -549,23 +543,15 @@ fundamental_trace(JSTracer *tracer,
  * tell, it would only be used if no constructor were provided to
  * JS_InitClass. The constructor from JS_InitClass is not applied to
  * the prototype unless JSCLASS_CONSTRUCT_PROTOTYPE is in flags.
- *
- * We allocate 1 reserved slot; this is typically unused, but if the
- * fundamental is for a nested structure inside a parent structure, the
- * reserved slot is used to hold onto the parent Javascript object and
- * make sure it doesn't get freed.
  */
-struct JSClass gjs_fundamental_instance_class = {
-    "GFundamental_Object",
-    JSCLASS_HAS_PRIVATE |
-    JSCLASS_IMPLEMENTS_BARRIERS,
+static const struct JSClassOps gjs_fundamental_class_ops = {
     NULL,  /* addProperty */
     NULL,  /* deleteProperty */
     NULL,  /* getProperty */
     NULL,  /* setProperty */
     NULL,  /* enumerate */
     fundamental_instance_resolve,
-    NULL,  /* convert */
+    nullptr,  /* mayResolve */
     fundamental_finalize,
     NULL,  /* call */
     NULL,  /* hasInstance */
@@ -573,6 +559,12 @@ struct JSClass gjs_fundamental_instance_class = {
     fundamental_trace
 };
 
+struct JSClass gjs_fundamental_instance_class = {
+    "GFundamental_Object",
+    JSCLASS_HAS_PRIVATE | JSCLASS_FOREGROUND_FINALIZE,
+    &gjs_fundamental_class_ops
+};
+
 static JSPropertySpec gjs_fundamental_instance_proto_props[] = {
     JS_PS_END
 };
@@ -620,7 +612,7 @@ gjs_lookup_fundamental_prototype(JSContext    *context,
         constructor = &value.toObject();
     }
 
-    g_assert(constructor != NULL);
+    g_assert(constructor);
 
     if (!gjs_object_get_property(context, constructor,
                                  GJS_STRING_PROTOTYPE, &value))
@@ -780,7 +772,7 @@ gjs_object_from_g_fundamental(JSContext    *context,
 
     object = JS_NewObjectWithGivenProto(context, JS_GetClass(proto), proto);
 
-    if (object == NULL)
+    if (!object)
         goto out;
 
     init_fundamental_instance(context, object);
@@ -822,7 +814,7 @@ gjs_g_fundamental_from_object(JSContext       *context,
 {
     FundamentalInstance *priv;
 
-    if (obj == NULL)
+    if (!obj)
         return NULL;
 
     priv = priv_from_js(context, obj);
