@@ -1,4 +1,6 @@
+/* exported _init, interfaces, properties, registerClass, requires, signals */
 // Copyright 2011 Jasper St. Pierre
+// Copyright 2017 Philip Chimento <philip.chimento@gmail.com>, <philip@endlessm.com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -18,12 +20,75 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 // IN THE SOFTWARE.
 
-const Lang = imports.lang;
 const Gi = imports._gi;
 const CjsPrivate = imports.gi.CjsPrivate;
+const Legacy = imports._legacy;
 
 let GObject;
 
+var GTypeName = Symbol('GType name');
+var interfaces = Symbol('GObject interfaces');
+var properties = Symbol('GObject properties');
+var requires = Symbol('GObject interface requires');
+var signals = Symbol('GObject signals');
+
+// These four will be aliased to GTK
+var _children = Symbol('GTK widget template children');
+var _cssName = Symbol('GTK widget CSS name');
+var _internalChildren = Symbol('GTK widget template internal children');
+var _template = Symbol('GTK widget template');
+
+function registerClass(klass) {
+    if (arguments.length == 2) {
+        // The two-argument form is the convenient syntax without ESnext
+        // decorators and class data properties. The first argument is an
+        // object with meta info such as properties and signals. The second
+        // argument is the class expression for the class itself.
+        //
+        //     var MyClass = GObject.registerClass({
+        //         Properties: { ... },
+        //         Signals: { ... },
+        //     }, class MyClass extends GObject.Object {
+        //         _init() { ... }
+        //     });
+        //
+        // When decorators and class data properties become part of the JS
+        // standard, this function can be used directly as a decorator.
+        let metaInfo = arguments[0];
+        klass = arguments[1];
+        if ('GTypeName' in metaInfo)
+            klass[GTypeName] = metaInfo.GTypeName;
+        if ('Implements' in metaInfo)
+            klass[interfaces] = metaInfo.Implements;
+        if ('Properties' in metaInfo)
+            klass[properties] = metaInfo.Properties;
+        if ('Signals' in metaInfo)
+            klass[signals] = metaInfo.Signals;
+        if ('Requires' in metaInfo)
+            klass[requires] = metaInfo.Requires;
+        if ('CssName' in metaInfo)
+            klass[_cssName] = metaInfo.CssName;
+        if ('Template' in metaInfo)
+            klass[_template] = metaInfo.Template;
+        if ('Children' in metaInfo)
+            klass[_children] = metaInfo.Children;
+        if ('InternalChildren' in metaInfo)
+            klass[_internalChildren] = metaInfo.InternalChildren;
+    }
+
+    if (!(klass.prototype instanceof GObject.Object) &&
+        !(klass.prototype instanceof GObject.Interface))
+        throw new TypeError('GObject.registerClass() used with invalid base ' +
+            `class (is ${Object.getPrototypeOf(klass).name})`);
+
+    // Find the "least derived" class with a _classInit static function; there
+    // definitely is one, since this class must inherit from GObject
+    let initclass = klass;
+    while (typeof initclass._classInit === 'undefined')
+        initclass = Object.getPrototypeOf(initclass.prototype).constructor;
+    return initclass._classInit(klass);
+}
+
 // Some common functions between GObject.Class and GObject.Interface
 
 function _createSignals(gtype, signals) {
@@ -42,198 +107,68 @@ function _createSignals(gtype, signals) {
     }
 }
 
-function _createGTypeName(params) {
-    if (params.GTypeName)
-        return params.GTypeName;
-    else
-        return 'Gjs_' + params.Name;
+function _createGTypeName(klass) {
+    if (klass.hasOwnProperty(GTypeName))
+        return klass[GTypeName];
+    return `Gjs_${klass.name}`;
 }
 
-function _getGObjectInterfaces(interfaces) {
-    return interfaces.filter((iface) => iface.hasOwnProperty('$gtype'));
-}
-
-function _propertiesAsArray(params) {
+function _propertiesAsArray(klass) {
     let propertiesArray = [];
-    if (params.Properties) {
-        for (let prop in params.Properties) {
-            propertiesArray.push(params.Properties[prop]);
+    if (klass.hasOwnProperty(properties)) {
+        for (let prop in klass[properties]) {
+            propertiesArray.push(klass[properties][prop]);
         }
     }
     return propertiesArray;
 }
 
-const GObjectMeta = new Lang.Class({
-    Name: 'GObjectClass',
-    Extends: Lang.Class,
-
-    _init: function (params) {
-        // retrieve signals and remove them from params before chaining
-	let signals = params.Signals;
-        delete params.Signals;
-
-        this.parent(params);
-
-        if (signals)
-            _createSignals(this.$gtype, signals);
-
-	let propertyObj = { };
-	Object.getOwnPropertyNames(params).forEach(function(name) {
-            if (name == 'Name' || name == 'Extends' || name == 'Abstract')
-		return;
-
-            let descriptor = Object.getOwnPropertyDescriptor(params, name);
-
-            if (typeof descriptor.value === 'function') {
-		let wrapped = this.prototype[name];
-
-                if (name.slice(0, 6) == 'vfunc_') {
-                    Gi.hook_up_vfunc(this.prototype, name.slice(6), wrapped);
-                } else if (name.slice(0, 3) == 'on_') {
-                    let id = GObject.signal_lookup(name.slice(3).replace('_', '-'), this.$gtype);
-                    if (id != 0) {
-                        GObject.signal_override_class_closure(id, this.$gtype, function() {
-                            let argArray = Array.prototype.slice.call(arguments);
-                            let emitter = argArray.shift();
-
-                            return wrapped.apply(emitter, argArray);
-                        });
-                    }
-                }
-	    }
-	}.bind(this));
-    },
-
-    _isValidClass: function(klass) {
-        let proto = klass.prototype;
-
-        if (!proto)
-            return false;
-
-        // If proto == GObject.Object.prototype, then
-        // proto.__proto__ is Object, so "proto instanceof GObject.Object"
-        // will return false.
-        return proto == GObject.Object.prototype ||
-            proto instanceof GObject.Object;
-    },
-
-    // If we want an object with a custom JSClass, we can't just
-    // use a function. We have to use a custom constructor here.
-    _construct: function(params) {
-        if (!params.Name)
-            throw new TypeError("Classes require an explicit 'Name' parameter.");
-        let name = params.Name;
-
-        let gtypename = _createGTypeName(params);
-
-        if (!params.Extends)
-            params.Extends = GObject.Object;
-        let parent = params.Extends;
-
-        if (!this._isValidClass(parent))
-            throw new TypeError('GObject.Class used with invalid base class (is ' + parent + ')');
-
-        let interfaces = params.Implements || [];
-        if (parent instanceof Lang.Class)
-            interfaces = interfaces.filter((iface) => !parent.implements(iface));
-        let gobjectInterfaces = _getGObjectInterfaces(interfaces);
-
-        let propertiesArray = _propertiesAsArray(params);
-        delete params.Properties;
-
-        let newClassConstructor = Gi.register_type(parent.prototype, gtypename,
-            gobjectInterfaces, propertiesArray);
-
-        let newClass = __private_GjsConstructorProxy(newClassConstructor,
-            this.constructor.prototype);
-
-        newClass.__super__ = parent;
-        newClass.prototype.constructor = newClass;
-
-        newClass._init.apply(newClass, arguments);
-
-        Object.defineProperties(newClass.prototype, {
-            '__metaclass__': { writable: false,
-                               configurable: false,
-                               enumerable: false,
-                               value: this.constructor },
-            '__interfaces__': { writable: false,
-                                configurable: false,
-                                enumerable: false,
-                                value: interfaces }
-        });
-
-        interfaces.forEach((iface) => {
-            if (iface instanceof Lang.Interface)
-                iface._check(newClass.prototype);
-        });
-
-        return newClass;
-    },
-
-    // Overrides Lang.Class.implements()
-    implements: function (iface) {
-        if (iface instanceof GObject.Interface) {
-            return GObject.type_is_a(this.$gtype, iface.$gtype);
-        } else {
-            return this.parent(iface);
-        }
-    }
-});
-
-function GObjectInterface(params) {
-    return this._construct.apply(this, arguments);
+function _copyAllDescriptors(target, source) {
+    Object.getOwnPropertyNames(source)
+    .filter(key => !['prototype', 'constructor'].includes(key))
+    .concat(Object.getOwnPropertySymbols(source))
+    .forEach(key => {
+        let descriptor = Object.getOwnPropertyDescriptor(source, key);
+        Object.defineProperty(target, key, descriptor);
+    });
 }
 
-GObjectMeta.MetaInterface = GObjectInterface;
+function _interfacePresent(required, klass) {
+    if (!klass[interfaces])
+        return false;
+    if (klass[interfaces].includes(required))
+        return true;  // implemented here
+    // Might be implemented on a parent class
+    return _interfacePresent(required, Object.getPrototypeOf(klass));
+}
 
-GObjectInterface.__super__ = Lang.Interface;
-GObjectInterface.prototype = Object.create(Lang.Interface.prototype);
-GObjectInterface.prototype.constructor = GObjectInterface;
-GObjectInterface.prototype.__name__ = 'GObjectInterface';
+function _checkInterface(iface, proto) {
+    // Check that proto implements all of this interface's required interfaces.
+    // "proto" refers to the object's prototype (which implements the interface)
+    // whereas "iface.prototype" is the interface's prototype (which may still
+    // contain unimplemented methods.)
+    if (typeof iface[requires] === 'undefined')
+        return;
 
-GObjectInterface.prototype._construct = function (params) {
-    if (!params.Name) {
-        throw new TypeError("Interfaces require an explicit 'Name' parameter.");
+    let unfulfilledReqs = iface[requires].filter(required => {
+        // Either the interface is not present or it is not listed before the
+        // interface that requires it or the class does not inherit it. This is
+        // so that required interfaces don't copy over properties from other
+        // interfaces that require them.
+        let ifaces = proto.constructor[interfaces];
+        return ((!_interfacePresent(required, proto.constructor) ||
+            ifaces.indexOf(required) > ifaces.indexOf(iface)) &&
+            !(proto instanceof required));
+    }).map(required =>
+        // required.name will be present on JS classes, but on introspected
+        // GObjects it will be the C name. The alternative is just so that
+        // we print something if there is garbage in Requires.
+        required.name || required);
+    if (unfulfilledReqs.length > 0) {
+        throw new Error('The following interfaces must be implemented before ' +
+            `${iface.name}: ${unfulfilledReqs.join(', ')}`);
     }
-
-    let gtypename = _createGTypeName(params);
-    delete params.GTypeName;
-
-    let interfaces = params.Requires || [];
-    let gobjectInterfaces = _getGObjectInterfaces(interfaces);
-
-    let properties = _propertiesAsArray(params);
-    delete params.Properties;
-
-    let newInterfaceConstructor = Gi.register_interface(gtypename,
-        gobjectInterfaces, properties);
-
-    let newInterface = __private_GjsConstructorProxy(newInterfaceConstructor,
-       this.constructor.prototype);
-    newInterface.__super__ = GObjectInterface;
-    newInterface.prototype.constructor = newInterface;
-
-    newInterface._init.apply(newInterface, arguments);
-
-    Object.defineProperty(newInterface.prototype, '__metaclass__', {
-        writable: false,
-        configurable: false,
-        enumerable: false,
-        value: this.constructor
-    });
-
-    return newInterface;
-};
-
-GObjectInterface.prototype._init = function (params) {
-    let signals = params.Signals;
-    delete params.Signals;
-
-    Lang.Interface.prototype._init.call(this, params);
-
-    _createSignals(this.$gtype, signals);
-};
+}
 
 function _init() {
 
@@ -383,7 +318,7 @@ function _init() {
                          get: function() { return CjsPrivate.param_spec_get_owner_type(this) } },
     });
 
-
+    let {GObjectMeta, GObjectInterface} = Legacy.defineGObjectLegacyObjects(GObject);
     this.Class = GObjectMeta;
     this.Interface = GObjectInterface;
     this.Object.prototype.__metaclass__ = this.Class;
@@ -395,6 +330,117 @@ function _init() {
         return this;
     };
 
+    GObject.registerClass = registerClass;
+
+    GObject.Object._classInit = function(klass) {
+        let gtypename = _createGTypeName(klass);
+        let gobjectInterfaces = klass.hasOwnProperty(interfaces) ?
+            klass[interfaces] : [];
+        let propertiesArray = _propertiesAsArray(klass);
+        let parent = Object.getPrototypeOf(klass);
+        let gobjectSignals = klass.hasOwnProperty(signals) ?
+            klass[signals] : [];
+
+        let newClass = Gi.register_type(parent.prototype, gtypename,
+            gobjectInterfaces, propertiesArray);
+        Object.setPrototypeOf(newClass, parent);
+
+        _createSignals(newClass.$gtype, gobjectSignals);
+
+        _copyAllDescriptors(newClass, klass);
+        gobjectInterfaces.forEach(iface =>
+            _copyAllDescriptors(newClass.prototype, iface.prototype));
+        _copyAllDescriptors(newClass.prototype, klass.prototype);
+
+        Object.getOwnPropertyNames(newClass.prototype)
+        .filter(name => name.startsWith('vfunc_') || name.startsWith('on_'))
+        .forEach(name => {
+            let descr = Object.getOwnPropertyDescriptor(newClass.prototype, name);
+            if (typeof descr.value !== 'function')
+                return;
+
+            let func = newClass.prototype[name];
+
+            if (name.startsWith('vfunc_')) {
+                Gi.hook_up_vfunc(newClass.prototype, name.slice(6), func);
+            } else if (name.startsWith('on_')) {
+                let id = GObject.signal_lookup(name.slice(3).replace('_', '-'),
+                    newClass.$gtype);
+                if (id !== 0) {
+                    GObject.signal_override_class_closure(id, newClass.$gtype, function() {
+                        let argArray = Array.from(arguments);
+                        let emitter = argArray.shift();
+
+                        return func.apply(emitter, argArray);
+                    });
+                }
+            }
+        });
+
+        gobjectInterfaces.forEach(iface =>
+            _checkInterface(iface, newClass.prototype));
+
+        // For backwards compatibility only. Use instanceof instead.
+        newClass.implements = function(iface) {
+            if (iface.$gtype)
+                return GObject.type_is_a(newClass.$gtype, iface.$gtype);
+            return false;
+        }
+
+        return newClass;
+    };
+
+    GObject.Interface._classInit = function(klass) {
+        let gtypename = _createGTypeName(klass);
+        let gobjectInterfaces = klass.hasOwnProperty(requires) ?
+            klass[requires] : [];
+        let properties = _propertiesAsArray(klass);
+        let gobjectSignals = klass.hasOwnProperty(signals) ?
+            klass[signals] : [];
+
+        let newInterface = Gi.register_interface(gtypename, gobjectInterfaces,
+            properties);
+
+        _createSignals(newInterface.$gtype, gobjectSignals);
+
+        _copyAllDescriptors(newInterface, klass);
+
+        Object.getOwnPropertyNames(klass.prototype)
+        .filter(key => key !== 'constructor')
+        .concat(Object.getOwnPropertySymbols(klass.prototype))
+        .forEach(key => {
+            let descr = Object.getOwnPropertyDescriptor(klass.prototype, key);
+
+            // Create wrappers on the interface object so that generics work (e.g.
+            // SomeInterface.some_function(this, blah) instead of
+            // SomeInterface.prototype.some_function.call(this, blah)
+            if (typeof descr.value === 'function') {
+                let interfaceProto = klass.prototype;  // capture in closure
+                newInterface[key] = function () {
+                    return interfaceProto[key].call.apply(interfaceProto[key],
+                        arguments);
+                };
+            }
+
+            Object.defineProperty(newInterface.prototype, key, descr);
+        });
+
+        return newInterface;
+    };
+
+    /**
+     * Use this to signify a function that must be overridden in an
+     * implementation of the interface.
+     */
+    GObject.NotImplementedError = class NotImplementedError extends Error {
+        get name() { return 'NotImplementedError'; }
+    };
+
+    GObject._cssName = _cssName;
+    GObject._template = _template;
+    GObject._children = _children;
+    GObject._internalChildren = _internalChildren;
+
     // fake enum for signal accumulators, keep in sync with gi/object.c
     this.AccumulatorType = {
         NONE: 0,
